from pathlib import Path

from PIL import Image
from loguru import logger

import pcleaner.image_ops as ops
import pcleaner.structures as st


def denoise_page(d_data: st.DenoiserData) -> st.DenoiseAnalytic:
    """
    Load the MaskData from the json file and perform the denoising process.

    :param d_data: All the data needed for the denoising process.
    :return: Analytics.
    """
    # Load all the cached data.
    mask_data = st.MaskData.from_json(d_data.json_path.read_text(encoding="utf-8"))
    mask_image = Image.open(mask_data.mask_path)

    # Clobber protection prefixes have the form "{UUID}_file name", ex. d91d86d1-b8d2-400b-98b2-2d0337973631_0023.json
    clobber_protection_prefix = d_data.json_path.stem.split("_")[0]
    cache_out_path = (
        d_data.cache_dir
        / f"{clobber_protection_prefix}_{mask_data.original_path.with_suffix('.png').name}"
    )

    def save_mask(img, name_suffix) -> None:
        if d_data.show_masks:
            img.save(cache_out_path.with_stem(cache_out_path.stem + name_suffix))

    # Scale the mask to the original image size, if needed.
    cleaned_image = Image.open(mask_data.original_path)
    mask_image = mask_image.convert("LA")
    cleaned_image = cleaned_image.convert("RGB")
    if cleaned_image.size != mask_image.size:
        mask_image = mask_image.resize(cleaned_image.size, resample=Image.NEAREST)

    cleaned_image.paste(mask_image, (0, 0), mask_image)
    original_path: Path = mask_data.original_path

    # Alias.
    g_conf = d_data.general_config
    d_conf = d_data.denoiser_config

    # Filter for the min deviation to consider for denoising.
    boxes_to_denoise: list[st.Box] = [
        box
        for box, deviation in mask_data.boxes_with_deviation
        if deviation > d_conf.noise_min_standard_deviation
    ]

    noise_masks_with_coords: list[tuple[Image.Image, tuple[int, int]]] = [
        ops.generate_noise_mask(cleaned_image, mask_image, box, d_conf) for box in boxes_to_denoise
    ]

    if noise_masks_with_coords:
        combined_noise_mask = ops.combine_noise_masks(cleaned_image.size, noise_masks_with_coords)
        cleaned_image.paste(combined_noise_mask, (0, 0), combined_noise_mask)
    else:
        # noinspection PyTypeChecker
        combined_noise_mask = Image.new("LA", cleaned_image.size, (0, 0))

    save_mask(combined_noise_mask, "_noise_mask")
    save_mask(cleaned_image, "_clean_denoised")

    # Check if the output path is None. In that case we're only outputting to the cache directory.
    if d_data.output_dir is None:
        # # Check if we still need to output the isolated text, otherwise we're done.
        # if d_data.extract_text:
        #     # Extract the text layer from the image.
        #     logger.debug(f"Extracting text from {original_path}")
        #     base_image = Image.open(original_path)
        #     text_img = ops.extract_text(base_image, mask_image)
        #     save_mask(text_img, "_text")
        #
        # Package the analytics. We're only interested in the std deviations.
        return st.DenoiseAnalytic(
            tuple(deviation for _, deviation in mask_data.boxes_with_deviation), original_path
        )

    # Settle on the final output path for the cleaned image.
    if d_data.output_dir.is_absolute():
        final_out_path = d_data.output_dir / mask_data.target_path.name
    else:
        # Take the original image path, and place the image in a subdirectory.
        # This is for when multiple directories were passed in.
        final_out_path = (
            mask_data.target_path.parent / d_data.output_dir / mask_data.target_path.name
        )

    final_out_path.parent.mkdir(parents=True, exist_ok=True)
    final_cleaned_out_path = final_out_path.with_name(final_out_path.stem + "_clean.png")
    final_mask_out_path = final_out_path.with_name(final_out_path.stem + "_mask.png")
    final_mask_denoised_out_path = final_out_path.with_name(
        final_out_path.stem + "_denoised_mask.png"
    )

    # Check what the preferred output format is.
    if g_conf.preferred_file_type is None:
        # Use the original file type.
        final_cleaned_out_path = final_cleaned_out_path.with_suffix(original_path.suffix)
    else:
        final_cleaned_out_path = final_cleaned_out_path.with_suffix(g_conf.preferred_file_type)

    if g_conf.preferred_mask_file_type is None:
        # Use png by default.
        final_mask_out_path = final_mask_out_path.with_suffix(".png")
    else:
        final_mask_out_path = final_mask_out_path.with_suffix(g_conf.preferred_mask_file_type)

    # The arg parser should ensure that both can't be true at once, not like that'd be an issue, just plain silly.
    if not d_data.save_only_mask:
        # Save the final image.
        logger.debug(f"Saving final image to {final_cleaned_out_path}")
        ops.save_optimized(cleaned_image, final_cleaned_out_path, original_path)

    if not d_data.save_only_cleaned:
        # Save the final image.
        if d_data.separate_noise_masks:
            logger.debug(f"Saving final mask to {final_mask_out_path}")
            ops.save_optimized(mask_image, final_mask_out_path)

            logger.debug(f"Saving final denoised mask to {final_mask_denoised_out_path}")
            ops.save_optimized(combined_noise_mask, final_mask_denoised_out_path)
        else:
            # Combine both the mask and the denoised mask into one image.
            combined_noise_mask.paste(mask_image, (0, 0), mask_image)
            logger.debug(f"Saving final mask to {final_mask_out_path}")
            ops.save_optimized(combined_noise_mask, final_mask_out_path)

    if d_data.extract_text:
        # Extract the text layer from the image.
        logger.debug(f"Extracting text from {original_path}")
        base_image = Image.open(original_path)
        text_img = ops.extract_text(base_image, mask_image)
        text_out_path = final_out_path.with_name(final_out_path.stem + "_text.png")
        if g_conf.preferred_mask_file_type is None:
            # Use png by default.
            text_out_path = text_out_path.with_suffix(".png")
        else:
            text_out_path = text_out_path.with_suffix(g_conf.preferred_mask_file_type)
        ops.save_optimized(text_img, text_out_path, original_path)

    # Package the analytics. We're only interested in the std deviations.
    return st.DenoiseAnalytic(
        tuple(deviation for _, deviation in mask_data.boxes_with_deviation), original_path
    )



# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.5.3
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x02M\
\x00\
\x00\x0d\x94x\xda\xed\x97Mo\xe20\x10\x86\xef\xfe\x15\
\xfb\x03\xac*\x9e|\x90V\xca\xa1\x14\xba\xad\xb6\x8bV\
b\xd5\x1e\x10\x07o\x18 j\xb0+\xc7\xb4\xea\xfe\xfa\
\x9d|\x00\x86\xa4jU\xb1{X!\xc5\x87\xcck;\
\xef<3r\x92\xc9\x95\xce\xb5\x19\xce\xe7\x98\xda\xe2b\
\x90\x15\xf2W\x8e\xb3)\xab\xc2I\x18\xf1\xea\xaao/\
Wz\xadl\xe21gMu\xa7\xac\x91\x85\xdd\xc8g\
Q\xb8\x8d5\x93\x04\xbbU\x16U\x91\xd9\xd7\xed,'\
\xd6\xcc\x02\xc6&{vn\x95Lm\xf6\x8cdg)\
\xd5\x02\xc7\x98S<\xd3\xaavg\xcd\x1a\x1b\xa3B\x00\
\x17B\xd0\xf0\xf6\xbd\x9ey\x10\xee\xf9\x85\xb6_qh\
\x17\xd8P\x95\x1c\x92\xb9\xcc\x0bl{o9\xf76\xce\
\x8b\x8b\xfe\xdaZ\xad\xa6\xac/\xd3\xc7\x85\xa1\xf9\xb3\xcb\
\xdc\xa2Q\xd2b\x22\x22\x9f\x039\x85\xd0s\xf4\x916\
+\x99'\x10\xf4x3\xd8\x00Smd\x99\xe9\xb5N\
\xd7E\x12Qf\xbd\x80\x83\xef;\xd2\x8d~F\xe3J\
\xd7\xda`\xf3\xc8\x0a[\xb7\xb6\x81Z3\x83\x90\x0b?\
v\xe4\xbbL=&\x01-\x84\x98\x8b8t\x94\x11.\
d\xb5\x10D\xcc\xa3\x98\xc7\xfe\x9e\xb8&\x82e\x16\x11\
\x15!\xe2\x9e\xab\xd5\x09\xfa!\xf7c\x1e\x08G\xf9\xa1\
\x09b\xb9\xa5\x7f^9=\x8f\x1c\xf1>#\x11g\x89\
\x08C\x1e\x93\x1e\xc3\x8e\xf2\x95^=\xe5\xb8Be\xa5\
y\xed\x86\x0d=\xee{\xdc\x876\xe9\x008\xb9\xa4\x1a\
\xfc\x1b\xcc\xa5\x1c\x95\xf9\xf5\x0e1\x03ECj\x89\xc0\
?\x22f\x08\xcb\x06\xab\xc61H\xdf\xa0\x9c\xa1y\x03\
1m\x06\x81GCt\xf43\x90\x07 ,\x10\x9d\xfa\
\xf9\x83\x94'\xbb\x13\xaf\x937\xf54P\xd3\x80\xdf\xc5\
\xdb)\xc6\x89\xf7;\xbc\xb7o\x92O\x1c\xd4\x9d0\xff\
#\xce@\xbc\x9aq\x0c\xd4?\xb5\xcem\xf6\xf4\x99\x13\
\xe4\xf4F\xfc8\xe6\xfb\x0c_\xde`\xec`l3v\
\x8a}b\xfc\x0e\xe3\x87L\xcd\xf4\xcb\xe9l\xfeK\x94\
\xbf\xa2BzN\xf3\xf31N\x97\xf4\x81\x97\xf4\x0d\xe2\
o\xfcr\x97-\x96\x96\x8d\xe4a\xa4X\xd2\x8bs\xac\
\x8d\xa55\xeb\x95\xaa\xff\x09\xd8\xe4\xdb`8ei\xf3\
U\x9f\x04\x14y\xf8>e5\x9c]u\xf6\x8a\xd6\x88\
9v\xc6w\xd6w\xb9f\xaa\xbd\xa1S\xe8\xad\x5co\
\xd9\xa18\x9b\xba\x05\xfb\x03^\xd9Vy\
\x00\x00\x02&\
\x00\
\x00\x0d~x\xda\xed\x97]o\xda0\x14\x86\xef\xfd+\
\xf6\x03\xac*v\xe2\x10*\xf9\xa2\x14\xbaV\xdb\xd0$\
\xaa\xf6\x02q\xe1\x85\x03D\x04\xbbr\x9cV\xdd\xaf\xdf\
!\x1f`\x0a\x93\xa8\xbaJ\xbb\x88\x94\x5c\xe4\xbc\xc7\xce\
{\x1e[\xfe\x98^\x9b\xdc\xd8\xd1b\x01\xa9+.\x87\
Y\xa1~\xe50\x9f\x91*,EL\xab\xa7\xfe\xbc\xda\
\x98R;\x19\x10\xafM\xf5\xa5\x9dU\x85k\xe5\x8bX\
\xecbM\x12#w\xda\x81.2\xf7\xba\xcb\xf2bM\
\x16'dz`\xe7N\xab\xd4e\xcf\x80vVJ/\
a\x029\xc63\xa3kw\xce\x96\xd0\x18e\x8cS\xc6\
\x18\xbe\xc1\xa1\xd7\x8b\x80\x8b\x03\xbf\xfc\xd8/{k\x97\
\x93\x91\xder\x90\x0b\x95\x17p\xec\xfd\xc8y\xd0:/\
.\x07\xa5sF\xcf\xc8@\xa5\xeb\xa5\xc5\xfc\xf9U\xee\
\xc0j\xe5@\x86\x01Mz\xe81\xf6\xd4\xb1\xb1\x1b\x95\
\xcb\xa8OEDE\x9f\x0c!5Vm\x8b\xbc1i\
Y\xc8\x18\x8b\xeaE\x94\x87\xa1'\xdd\x9ag\xb0\xbet\
c,4\x7f\xab\x88\x9d\xd6Z\x9e\x92m\xe5\xb8\x8f)\
=O\xfe\x9e\xe9\xb5\xe4\x18\x15!\xe5\x91\xdfp\x0cK\
U5\xe4,\xa1qB\x93C\xb1Dx\xb9\xe4Q\xbc\
\xad\x8d\x06\xbeVW\xc7\x05\xa7\xcd\xeb\x89?\x0d\x22\xdc\
\xf6\x1a\xf6+\xb3\xfd\xd8\x13\x1f2\x14a.\x99\x104\
A=\xe1{\xc6\xd7f\xf3\x94\xc3\x06\xb4S\xf6\xf5\xfd\
\xa89E\xa7\x22\xe8P\x9f\x81\xfa\x16\xd4\x1c\xeci\xc6\
;\x90\xddl\xfe\x07\x88\xa7\xfb\xd5\xee$\xec\x16i7\
\x9f?\x02{\xb7\x85\xbc{\xd98I\xf3C\xa0O\x97\
v\x16i\xc4\xcc\x92\x84\xf6\xc4\xffK\xfa\xde\x98\xdce\
O\xdd\xd2\xf1\x89\x8c\x1f2x\xf9\xcbD\x164Lh\
\xc4\x8e\x01\xf3\x1e\xc5Y\x1e\xf2\x0e\xf0\x19\x80\x1f3=\
7/\xdd\x8a\xfc\x19\x88\xbf\x82\x06\xfcUs\xd7\x98\xa4\
+<\xd1\xc9\x81\x05\xf8\x0d_\x86\xca\xae\xc9X\xbd\x09\
\x14+\xdc)'\xc6:lQnt}\x01 \xd3o\
\xc3\xd1\x8c\xa4\xcd\x11^F\x18y\xfc1#5\x9d\xfd\
\xd8\xec\xc7\xabQr\xc0\xa0_L\x1d\xdf\xbb>\x103\
}\xdca;\xcc;\xad\xea\xd2\x1f\x8fV\xf1:\xf5\xe5\
?T\xf7RF\
"

qt_resource_name = b"\
\x00\x06\
\x07\xae\xc3\xc3\
\x00t\
\x00h\x00e\x00m\x00e\x00s\
\x00\x06\
\x06\x98\xbd\x05\
\x00b\
\x00r\x00e\x00e\x00z\x00e\
\x00\x0b\
\x00\xd0\xbf+\
\x00b\
\x00r\x00e\x00e\x00z\x00e\x00-\x00d\x00a\x00r\x00k\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00$\x00\x01\x00\x00\x00\x01\x00\x00\x02Q\
\x00\x00\x01\x8b\x0an\xba\x97\
\x00\x00\x00\x12\x00\x01\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x8b\x0an\xba\x97\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



import PySide6.QtGui as Qg
import PySide6.QtWidgets as Qw

from pcleaner.gui.ui_generated_files.ui_License import Ui_License


class LicenseDialog(Qw.QDialog, Ui_License):
    """
    Displays the license.
    """

    def __init__(
        self,
        parent: Qw.QWidget | None = None,
    ):
        """
        Initialize the widget.

        :param parent: The parent widget.
        """
        Qw.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setWindowIcon(Qg.QIcon(":/logo-tiny.png"))



from pathlib import Path

import PySide6.QtGui as Qg
import PySide6.QtWidgets as Qw

from pcleaner.gui.ui_generated_files.ui_NewProfile import Ui_NewProfile


class NewProfileDialog(Qw.QDialog, Ui_NewProfile):
    """
    The dialog for creating a new profile.
    It takes care of assigning a new file path.
    """

    default_path: Path
    protected_names: list[str]

    def __init__(
        self,
        default_path: Path,
        show_protection_hint: bool,
        protected_names: list[str],
    ) -> None:
        """
        Initialize the dialog.

        :param default_path: The default path to display.
        :param show_protection_hint: When true, show a hint about not being able to overwrite the default profile.
        :param protected_names: A list of names that are protected and cannot be used,
            as they are used for builtin profiles.
        """
        # For some reason, it doesn't want parent.
        Qw.QDialog.__init__(self)
        self.setupUi(self)

        self.default_path = default_path
        self.label_default_path.setText(str(self.default_path))
        self.protected_names = protected_names

        self.setWindowIcon(Qg.QIcon(":/logo-tiny.png"))

        # Ensure that the default path exists.
        self.default_path.mkdir(parents=True, exist_ok=True)

        self.label_warning_icon.setPixmap(Qg.QIcon.fromTheme("data-error").pixmap(16, 16))
        self.label_default_protection_hint.setVisible(show_protection_hint)

        self.validate()
        self.lineEdit_name.textChanged.connect(self.validate)
        self.lineEdit_location.textChanged.connect(self.validate)

        self.pushButton_browse_location.clicked.connect(self.browse_location)

        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

    def get_name(self) -> str:
        """
        Return the name of the new profile.

        :return: The name of the new profile.
        """
        return self.lineEdit_name.text().strip()

    def get_directory(self) -> Path:
        """
        Return the directory of the new profile.

        :return: The directory of the new profile.
        """
        return (
            self.default_path
            if self.radioButton_default_location.isChecked()
            else Path(self.lineEdit_location.text())
        )

    def get_save_path(self) -> Path:
        """
        Return the path where the new profile should be saved.

        :return: The path where the new profile should be saved.
        """
        return (self.get_directory() / self.get_name()).with_suffix(".conf")

    def browse_location(self) -> None:
        """
        Let the user choose a custom location. Only directories are allowed.
        """
        dialog = Qw.QFileDialog(self)
        dialog.setFileMode(Qw.QFileDialog.Directory)
        dialog.setOption(Qw.QFileDialog.ShowDirsOnly, True)
        dialog.setDirectory(str(self.default_path))

        if dialog.exec():
            self.lineEdit_location.setText(dialog.selectedFiles()[0])

    def validate(self) -> None:
        """
        Check if the current input is acceptable, and show warnings if not.
        """
        name = self.get_name()
        prospective_dir = self.get_directory()

        if not name:
            self.label_warning_message.setText("Please enter a name.")
            self.label_warning_message.show()
            self.label_warning_icon.hide()
            self.buttonBox.button(Qw.QDialogButtonBox.Save).setEnabled(False)
        elif name.lower() in self.protected_names:
            self.label_warning_message.setText("This name is used by a builtin profile.")
            self.label_warning_message.show()
            self.label_warning_icon.show()
            self.buttonBox.button(Qw.QDialogButtonBox.Save).setEnabled(False)
        elif (prospective_dir / name).with_suffix(".conf").exists():
            self.label_warning_message.setText("A profile with this name already exists.")
            self.label_warning_message.show()
            self.label_warning_icon.show()
            self.buttonBox.button(Qw.QDialogButtonBox.Save).setEnabled(False)
        else:
            self.label_warning_message.hide()
            self.label_warning_icon.hide()
            self.buttonBox.button(Qw.QDialogButtonBox.Save).setEnabled(True)

        if not prospective_dir.is_dir():
            self.label_warning_message.setText("The selected directory does not exist.")
            self.label_warning_message.show()
            self.label_warning_icon.show()
            self.buttonBox.button(Qw.QDialogButtonBox.Save).setEnabled(False)



import PySide6.QtCore as Qc
import PySide6.QtGui as Qg
import PySide6.QtWidgets as Qw
from PySide6.QtCore import Slot

from loguru import logger


ICON_SCALE_FACTOR = 0.75  # Adjust as desired, where 1.0 means same as badge size


class BadgeButton(Qw.QPushButton):
    """
    This is a button with a badge in the top right corner.
    This can be shown or hidden at any time.
    As of now, the badge is the refresh icon on top of a colored circle using the
    highlight color of the palette.
    Additionally, you can set the button to replace the normal background color with
    the highlight color when the button is checked (as these are checkable buttons).
    That's called accented focus.
    I did this to make the checked-ness more obvious, due to most of the button being covered
    mostly by a pixmap, leaving only the edges visible.
    """

    badge_visible: bool
    badge_icon: Qg.QIcon
    accent_color: Qg.QColor
    can_uncheck: bool = False
    accented_focus: bool = True

    badge_size: int = 20
    badge_margin: int = 4

    def __init__(self, parent=None) -> None:
        Qw.QPushButton.__init__(self, parent=parent)
        self.badge_visible = False
        self.badge_icon = Qg.QIcon.fromTheme("view-refresh")
        # Get the accent color from the palette.
        self.update_accent_color()

        self.clicked.connect(self.handle_toggle)
        # self.toggled.connect(self.updateButtonPalette)

    @Qc.Slot(bool)
    def handle_toggle(self, checked: bool) -> None:
        """
        Check if the button can be unchecked, otherwise preventing such an action.

        :param checked: The new checked state.
        """
        if not self.can_uncheck and not checked:
            self.setChecked(True)

    def set_badge_size(self, size: int, margin: int) -> None:
        """
        Set the size of the badge and the margin around it.
        """
        self.badge_size = size
        self.badge_margin = margin

    def paintEvent(self, event) -> None:
        # Let QPushButton handle its default painting.
        Qw.QPushButton.paintEvent(self, event)

        if self.badge_visible:
            painter = Qg.QPainter(self)
            # Draw the badge in the top right corner, with a margin.
            badge_rect = Qc.QRect(
                self.width() - self.badge_size - self.badge_margin,
                self.badge_margin,
                self.badge_size,
                self.badge_size,
            )

            # Draw the colored circle background for the badge.
            painter.setBrush(self.accent_color)
            painter.setPen(Qc.Qt.NoPen)
            painter.drawEllipse(badge_rect)

            # Calculate the size of the smaller icon rectangle.
            icon_size = int(self.badge_size * ICON_SCALE_FACTOR)
            pixmap = self.badge_icon.pixmap(Qc.QSize(icon_size, icon_size))

            # Calculate the top-left point for drawing the pixmap to center it.
            pixmap_x = badge_rect.center().x() - icon_size // 2
            pixmap_y = badge_rect.center().y() - icon_size // 2
            # Draw the centered icon pixmap.
            # Fuck this shit I'm hard-coding a 1px offset, no bloody clue why it's off by one otherwise.
            painter.drawPixmap(pixmap_x + 1, pixmap_y + 1, pixmap)

    def changeEvent(self, event: Qc.QEvent) -> None:
        super(BadgeButton, self).changeEvent(event)
        if event.type() == Qc.QEvent.PaletteChange:
            self.update_accent_color()
            self.update()

    @Slot(bool)
    def updateButtonPalette(self, checked) -> None:
        palette = self.palette()
        if checked:  # and self.accented_focus:
            palette.setColor(Qg.QPalette.Button, self.accent_color)
        else:
            # Reset to the default button color
            palette.setColor(
                Qg.QPalette.Button, palette.color(Qg.QPalette.Normal, Qg.QPalette.Button)
            )
        self.setPalette(palette)

    def update_accent_color(self) -> None:
        """
        Use the Highlight color, aka Accent color to draw the background of the badge.
        """
        self.accent_color = self.palette().color(Qg.QPalette.ColorRole.Highlight)
        if self.accented_focus:
            self.setStyleSheet(
                """
                QPushButton:checked {
                    background-color: palette(highlight);
                }
                """
            )

    def set_badge_visible(self, visible: bool) -> None:
        self.badge_visible = visible
        self.update()

    def hide_badge(self) -> None:
        self.set_badge_visible(False)

    def show_badge(self) -> None:
        self.set_badge_visible(True)



import io
import re
import sys
from io import StringIO
from pathlib import Path

import PySide6.QtCore as Qc
import PySide6.QtGui as Qg
import PySide6.QtWidgets as Qw
from loguru import logger


# For all show functions, pad the dialog message, so that the dialog is not too narrow for the window title.
MIN_MSG_LENGTH = 50


class SelectableMessageBox(Qw.QMessageBox):
    """
    Subclass the QMessageBox to make the text selectable.
    """

    def __init__(self, *args, **kwargs):
        super(SelectableMessageBox, self).__init__(*args, **kwargs)
        self.setTextInteractionFlags(Qc.Qt.TextSelectableByMouse)

        labels = self.findChildren(Qw.QLabel)
        for label in labels:
            label.setTextInteractionFlags(Qc.Qt.TextSelectableByMouse)


def show_critical(parent, title: str, msg: str) -> int:
    msg = msg.ljust(MIN_MSG_LENGTH)
    box = SelectableMessageBox(
        Qw.QMessageBox.Critical, title, msg, Qw.QMessageBox.Yes | Qw.QMessageBox.Abort, parent
    )
    return box.exec()


def show_warning(parent, title: str, msg: str) -> None:
    msg = msg.ljust(MIN_MSG_LENGTH)
    box = SelectableMessageBox(Qw.QMessageBox.Warning, title, msg, Qw.QMessageBox.Ok, parent)
    box.exec()


def show_info(parent, title: str, msg: str) -> None:
    msg = msg.ljust(MIN_MSG_LENGTH)
    box = SelectableMessageBox(Qw.QMessageBox.Information, title, msg, Qw.QMessageBox.Ok, parent)
    box.exec()


def show_question(
    parent, title: str, msg: str, buttons=Qw.QMessageBox.Yes | Qw.QMessageBox.Cancel
) -> int:
    msg = msg.ljust(MIN_MSG_LENGTH)
    dlg = Qw.QMessageBox(parent)
    dlg.setWindowTitle(title)
    dlg.setText(msg)
    dlg.setStandardButtons(buttons)
    dlg.setIcon(Qw.QMessageBox.Question)
    return dlg.exec()


def open_file(path: Path) -> None:
    """
    Open any given file with the default application.
    """
    logger.info(f"Opening file {path}")
    try:
        # Use Qt to open the file, so that it works on all platforms.
        Qg.QDesktopServices.openUrl(Qc.QUrl.fromLocalFile(str(path)))
    except Exception as e:
        logger.exception(e)


class CaptureOutput(Qc.QObject):
    text_written = Qc.Signal(str, str)

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self._io_stdout = io.StringIO()
        self._io_stderr = io.StringIO()

    def __enter__(self) -> "CaptureOutput":
        self._original_stdout = sys.stdout
        self._original_stderr = sys.stderr
        sys.stdout = self
        sys.stderr = self
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        sys.stdout = self._original_stdout
        sys.stderr = self._original_stderr

    def write(self, string, stream="stdout") -> None:
        if stream == "stdout":
            self._io_stdout.write(string)
        elif stream == "stderr":
            self._io_stderr.write(string)
        self.text_written.emit(string, stream)

    def getvalue(self, stream="stdout") -> str:
        if stream == "stdout":
            return self._io_stdout.getvalue()
        elif stream == "stderr":
            return self._io_stderr.getvalue()

    def flush(self) -> None:
        self._io_stdout.flush()
        self._io_stderr.flush()


# Mapping between KDE color keys and QPalette roles
section_role_mapping = {
    "Colors:Button": {
        "BackgroundNormal": Qg.QPalette.Button,
        "ForegroundNormal": Qg.QPalette.ButtonText,
        "ForegroundActive": Qg.QPalette.BrightText,
    },
    "Colors:View": {
        "BackgroundNormal": Qg.QPalette.Base,
        "BackgroundAlternate": Qg.QPalette.AlternateBase,
        "ForegroundNormal": Qg.QPalette.Text,
        "ForegroundLink": Qg.QPalette.Link,
        "ForegroundVisited": Qg.QPalette.LinkVisited,
        "ForegroundInactive": Qg.QPalette.PlaceholderText,
    },
    "Colors:Selection": {
        "BackgroundNormal": Qg.QPalette.Highlight,
        "ForegroundNormal": Qg.QPalette.HighlightedText,
    },
    "Colors:Tooltip": {
        "BackgroundNormal": Qg.QPalette.ToolTipBase,
        "ForegroundNormal": Qg.QPalette.ToolTipText,
    },
    "Colors:Window": {
        "BackgroundNormal": Qg.QPalette.Window,
        "ForegroundNormal": Qg.QPalette.WindowText,
    },
    # Add more mappings as needed
}


def load_color_palette(theme: str) -> Qg.QPalette:
    """
    Provide a theme name and get a QPalette object.
    The name should match one of the files in the themes folder.

    :param theme: The name of the theme.
    :return: A QPalette object.
    """
    palette = Qg.QPalette()

    file = Qc.QFile(f":/themes/{theme}")
    if file.open(Qc.QFile.ReadOnly | Qc.QFile.Text):
        stream = Qc.QTextStream(file)
        content = stream.readAll()

        section = None
        for line in content.split("\n"):
            line = line.strip()
            if line.startswith("[") and line.endswith("]"):
                section = line[1:-1]
            elif "=" in line:
                key, value = map(str.strip, line.split("=", 1))
                role_mapping = section_role_mapping.get(section, {})
                qt_color_role = role_mapping.get(key, None)
                if qt_color_role is not None:
                    r, g, b = map(int, value.split(","))
                    palette.setColor(qt_color_role, Qg.QColor(r, g, b))
    else:
        raise ValueError(f"Could not open theme file: {theme}")

    # Fallback calculations
    if not palette.color(Qg.QPalette.Light).isValid():
        base = palette.color(Qg.QPalette.Button)
        palette.setColor(Qg.QPalette.Light, base.lighter(150))

    if not palette.color(Qg.QPalette.Dark).isValid():
        base = palette.color(Qg.QPalette.Window)
        palette.setColor(Qg.QPalette.Dark, base.darker(150))

    if not palette.color(Qg.QPalette.Midlight).isValid():
        light = palette.color(Qg.QPalette.Light)
        button = palette.color(Qg.QPalette.Button)
        midlight = Qg.QColor(
            (light.red() + button.red()) // 2,
            (light.green() + button.green()) // 2,
            (light.blue() + button.blue()) // 2,
        )
        palette.setColor(Qg.QPalette.Midlight, midlight)

    if not palette.color(Qg.QPalette.Mid).isValid():
        dark = palette.color(Qg.QPalette.Dark)
        button = palette.color(Qg.QPalette.Button)
        mid = Qg.QColor(
            (dark.red() + button.red()) // 2,
            (dark.green() + button.green()) // 2,
            (dark.blue() + button.blue()) // 2,
        )
        palette.setColor(Qg.QPalette.Mid, mid)

    if not palette.color(Qg.QPalette.Shadow).isValid():
        palette.setColor(Qg.QPalette.Shadow, Qg.QColor(0, 0, 0))

    return palette


def ansi_to_html(input_text: str) -> str:
    """
    Convert ansi escape sequences to html.

    Analytics HTML template
    <html><head><meta name="qrichtext" content="1" /><style type="text/css">
    p, li { white-space: pre-wrap; font-family: Noto Mono, Monospace; size: 12pt}
    </style></head><body>
    <p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px;
     -qt-block-indent:0; text-indent:0px;"><span style=" ">Mask Fitment Analytics
     <br />---------------------- <br />Total boxes: 5 | Masks succeeded: 5 (100%)
     | Masks failed: </span><span style="color:#b21818;">0</span><span style=" ">
     <br />Perfect masks: </span><span style="color:#18b2b2;">5</span><span style=" ">
     (100%) | Average border deviation: 0.00 <br /><br />Mask usage by mask size
     (smallest to largest): <br />Mask 0  :  </span><span style="color:#18b2b2;">0</span>
     <span style=" "> / 0 <br />Mask 1  :  </span><span style="color:#18b2b2;">0</span>
     <span style=" "> / 0 <br />Mask 2  :  </span><span style="color:#18b2b2;">0</span>
     <span style=" "> / 0 <br />Mask 3  :  </span><span style="color:#18b2b2;">0</span>
     <span style=" "> / 0 <br />Mask 4  :  </span><span style="color:#18b2b2;">0</span>
     <span style=" "> / 0 <br />Mask 5  :  </span><span style="color:#18b2b2;">0</span>
     <span style=" "> / 0 <br />Mask 6  : </span><span style="color:#18b2b2;">
     ████████████████████████████████████████</span><span style=" "> </span>
     <span style="color:#18b2b2;">1</span><span style=" "> / 1 <br />Mask 7  :  </span>
     <span style="color:#18b2b2;">0</span><span style=" "> / 0 <br />Mask 8  :  </span>
     <span style="color:#18b2b2;">0</span><span style=" "> / 0 <br />Mask 9  :  </span>
     <span style="color:#18b2b2;">0</span><span style=" "> / 0 <br />Mask 10 :  </span>
     <span style="color:#18b2b2;">0</span><span style=" "> / 0 <br />Box mask: </span>
     <span style="color:#18b2b2;">████████████████████████████████████████</span>
     <span style=" "> </span><span style="color:#18b2b2;">4</span><span style=" ">
     / 4 <br /><br /></span><span style="color:#18b2b2;">█ Perfect</span><span style=" ">
     | █ Total<br /><br /></span></p></body></html>
    """
    # Define ANSI escape sequences and corresponding Qt HTML color codes
    ansi_to_html_lut = {
        "31": "color:#eb514f;",
        "32": "color:#00ff00;",
        "33": "color:#ffff00;",
        "34": "color:#0000ff;",
        "35": "color:#a771bf;",
        "36": "color:#00a3a3;",
        "37": "color:#ffffff;",
        "39": " ",  # Default color
    }

    # Initialize HTML header and footer
    html_header = """<html><head><meta name="qrichtext" content="1" /><style type="text/css">
        p, li { white-space: pre-wrap; font-family: Noto Mono, Monospace; size: 12pt}
        </style></head><body>
        <p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">"""
    html_footer = "</p></body></html>"

    # Initialize output HTML with header
    buffer = StringIO()
    buffer.write(html_header)

    # Initialize current_style
    current_style = ""

    # Strip leading and trailing whitespace.
    input_text = input_text.strip()

    # Use regex to find all ANSI sequences and split text
    for plain_text, ansi_code in re.findall(r"(.*?)(?:\x1b\[(\d+)m|$)", input_text, re.DOTALL):
        # Add plain text
        newline = "\n"
        buffer.write(
            f'<span style="{current_style}">{plain_text.replace(newline, "<br />")}</span>'
        )

        # Update the style for the next span based on ANSI code
        if ansi_code:
            current_style = ansi_to_html_lut.get(ansi_code, "")

    # Add HTML footer
    buffer.write(html_footer)

    return buffer.getvalue()



import os
import platform
import sys
from io import StringIO

import PySide6
import PySide6.QtGui as Qg
import PySide6.QtWidgets as Qw
from loguru import logger
import torch

import pcleaner.cli_utils as cu
from pcleaner import __display_name__, __version__
from pcleaner.gui.mainwindow_driver import MainWindow


# This import is needed to load the icons.
import pcleaner.gui.rc_generated_files.rc_icons
import pcleaner.gui.rc_generated_files.rc_theme_icons
import pcleaner.gui.rc_generated_files.rc_themes


def launch() -> None:
    """
    Launch the GUI.
    """

    cu.get_log_path().parent.mkdir(parents=True, exist_ok=True)
    # Log up to 1MB to the log file.
    # loguru.logfile(str(cu.get_log_path()), maxBytes=2**30, backupCount=1, loglevel=loguru.DEBUG)

    # Set up file logging.
    logger.add(
        str(cu.get_log_path()),
        rotation="10 MB",
        retention="1 week",
        level="DEBUG"
    )

    logger.info("\n---- Starting up ----")
    buffer = StringIO()
    buffer.write("\n- Program Information -\n")
    buffer.write(f"Program: {__display_name__} {__version__}\n")
    buffer.write(f"Log file is {cu.get_log_path()}\n")
    buffer.write(f"Config file is {cu.get_config_path()}\n")
    buffer.write(f"Cache directory is {cu.get_cache_path()}\n")
    buffer.write("- System Information -\n")
    buffer.write(f"Operating System: {platform.system()} {platform.release()}\n")
    buffer.write(f"Machine: {platform.machine()}\n")
    buffer.write(f"Python Version: {sys.version}\n")
    buffer.write(f"PySide (Qt) Version: {PySide6.__version__}\n")
    buffer.write(f"Available Qt Themes: {', '.join(Qw.QStyleFactory.keys())}\n")
    buffer.write(f"CPU Cores: {os.cpu_count()}\n")
    if torch.cuda.is_available():
        buffer.write(f"GPU: {torch.cuda.get_device_name(0)} (CUDA enabled)\n")
    else:
        buffer.write("GPU: None (CUDA not available)\n")

    logger.info(buffer.getvalue())

    # Start the main window.
    app = Qw.QApplication(sys.argv)

    Qg.QIcon.setFallbackSearchPaths([":/icons", ":/icon-themes"])
    # We need to set an initial theme on Windows, otherwise the icons will fail to load
    # later on, even when switching the theme again.
    if platform.system() == "Windows":
        Qg.QIcon.setThemeName("breeze")
        Qg.QIcon.setThemeSearchPaths([":/icons", ":/icon-themes"])

    try:
        window = MainWindow()
        window.show()
        sys.exit(app.exec())
    except Exception as e:
        logger.exception(
            e,
        )
    finally:
        logger.info("---- Shutting down ----\n")


if __name__ == "__main__":
    launch()



import itertools
import shutil
from collections import defaultdict
from io import StringIO
from pathlib import Path
from typing import Sequence

import colorama as clr
from attrs import frozen
from natsort import natsorted

import pcleaner.helpers as hp
import pcleaner.structures as st


@frozen
class OCRAnalytic:
    num_boxes: int
    small_box_sizes: Sequence[int]
    removed_box_sizes: Sequence[int]


# I'll admit this file is a mess.
# These bar chart creators should be abstracted into a single implementation, not 3 different ones.


def terminal_width() -> int:
    # Use a fallback of 100 for unsupported terminals.
    width = shutil.get_terminal_size((100, 100)).columns
    # Make sure the width is at least 20, to avoid weirdness.
    return max(width, 20)


def show_ocr_analytics(
    analytics: Sequence[st.OCRAnalytic],
    max_ocr_size: int,
    max_columns: int = 100,
) -> str:
    """
    The analytics are gathered from the OCR pre-processing which has the goal of
    finding small boxes and to check if they contain text worth cleaning.

    The analytics for each page in the list consist of the following:
    - The number of boxes on the page.
    - The sizes of the small boxes.
    - The sizes of the boxes that were removed.
    - list of: The original file path and the removed text and the box it belonged to.

    Various statistics are then shown.

    :param analytics: The analytics gathered from each pre-processed page.
    :param max_ocr_size: The maximum size of a box to be considered small.
    :param max_columns: The maximum number of columns to use for the chart per line.
    :return: The analytics as a string.
    """
    buffer = StringIO()

    num_boxes = sum(a.num_boxes for a in analytics)
    num_small_boxes = sum(len(a.box_sizes_ocr) for a in analytics)
    small_box_sizes = list(itertools.chain.from_iterable(a.box_sizes_ocr for a in analytics))
    removed_box_sizes = list(itertools.chain.from_iterable(a.box_sizes_removed for a in analytics))
    small_box_ratio = f"{num_small_boxes / num_boxes:.0%}" if num_boxes > 0 else "N/A"
    removed_box_ratio = f"{len(removed_box_sizes) / num_boxes:.0%}" if num_boxes > 0 else "N/A"
    removed_among_small_ratio = (
        f"{len(removed_box_sizes) / num_small_boxes:.0%}" if num_small_boxes > 0 else "N/A"
    )
    # Partition the small box sizes into 5 rough size ranges, equally spaced.
    part_size = 500 if max_ocr_size < 500 * 10 else max_ocr_size // 10
    partitioned_small_box_sizes = partition_list(
        small_box_sizes, partition_size=part_size, max_value=max_ocr_size
    )
    partitioned_removed_boxes = partition_list(
        removed_box_sizes, partition_size=part_size, max_value=max_ocr_size
    )
    buffer.write("\nOCR Analytics\n")
    buffer.write("-------------\n")
    buffer.write(
        f"Number of boxes: {num_boxes} | "
        f"Number of small boxes: {num_small_boxes} ({small_box_ratio})\n"
    )
    buffer.write(
        f"Number of removed boxes: {len(removed_box_sizes)} ({removed_box_ratio} total, {removed_among_small_ratio} of small boxes)\n"
    )
    if small_box_sizes:
        buffer.write("\nSmall box sizes:\n")
        buffer.write(
            draw_pretty_ocr_result_chart(
                partitioned_small_box_sizes, partitioned_removed_boxes, max_columns
            )
            + "\n"
        )
    else:
        buffer.write("No not-removed small boxes found.\n")

    # Show removed texts.
    removed_path_texts: list[tuple[Path, str, st.Box]] = list(
        itertools.chain.from_iterable(a.removed_box_data for a in analytics)
    )

    if removed_path_texts:
        # Find and then remove the longest common prefix from the file paths.
        paths, texts, _ = zip(*removed_path_texts)
        removed_paths = hp.trim_prefix_from_paths(paths)
        removed_path_texts = list(zip(removed_paths, texts))
        # Sort by file path.
        removed_path_texts = natsorted(removed_path_texts, key=lambda p: p[0])

    if removed_path_texts:
        buffer.write("\nRemoved bubbles:\n")
        for path, text in removed_path_texts:
            buffer.write(f"Page {path}: {text}\n")

    buffer.write("\n")

    return buffer.getvalue()


def draw_pretty_ocr_result_chart(
    small_box_data: list[tuple[str, int]],
    removed_box_data: list[tuple[str, int]],
    max_columns: int = 100,
) -> str:
    """
    Draw a pretty stacked bar chart.

    :param small_box_data: The data for the small boxes.
    :param removed_box_data: The data for the removed boxes.
    :param max_columns: The maximum number of columns to use for the chart per line.
    :return: The chart as a string.
    """
    buffer = StringIO()

    data_array = [
        (label, lower, upper)
        for (label, lower), (_, upper) in zip(small_box_data, removed_box_data)
    ]
    max_label_width = max(len(label) for label, _, _ in data_array)
    bar_width = (
        max_columns - max_label_width - 16
    )  # 16 for the spacing and trailing number and padding.
    max_value = max(lower + upper for _, lower, upper in data_array)

    for label, lower, upper in data_array:
        lower_bar = "█" * int((lower - upper) / max_value * bar_width)
        upper_bar = "█" * int(upper / max_value * bar_width)
        buffer.write(
            f"{label:<{max_label_width}}: "
            f"{lower_bar}{clr.Fore.RED}{upper_bar} {upper}{clr.Style.RESET_ALL} / {lower}\n"
        )

    # Show legend.
    buffer.write(f"\n█ Small boxes | {clr.Fore.RED}█ Removed boxes{clr.Style.RESET_ALL}")

    return buffer.getvalue()


def draw_masker_histogram(data: dict[str, tuple[int, int]], max_columns: int = 100) -> str:
    """
    Draw a histogram of the masker results.

    :param data: A dict of the mask name, (perfect uses, all uses).
    :param max_columns: The maximum number of columns to use for the histogram per line.
    :return: The histogram as a string.
    """

    # Find the maximum value in the data. (at least 1 to avoid division by zero)
    max_value = max(list(map(lambda x: x[1], data.values())))
    max_value = max(max_value, 1)

    # Longest key.
    max_label_width = max(len(k) for k in data.keys())
    bar_width = (
        max_columns - max_label_width - 16
    )  # 16 for the spacing and trailing number and padding.

    buffer = StringIO()

    # Draw the histogram.
    for key, value in data.items():
        # Use a Unicode character for the bar.
        left_bar = "█" * int(value[0] / max_value * bar_width)
        right_bar = "█" * int((value[1] - value[0]) / max_value * bar_width)
        # If both are 0 in length, but one of them isn't 0 in value, draw the right-most one as a sliver.
        if not left_bar and not right_bar and value[1] > value[0]:
            right_bar = "▏"
        elif not left_bar and not right_bar and value[0] > 0:
            left_bar = "▏"

        buffer.write(
            f"{key:<{max_label_width}}: "
            f"{clr.Fore.CYAN}{left_bar}{clr.Style.RESET_ALL}{right_bar} "
            f"{clr.Fore.CYAN}{value[0]}{clr.Fore.RESET} / {value[1]}\n"
        )

    # Show legend.
    buffer.write(f"\n{clr.Fore.CYAN}█ Perfect{clr.Style.RESET_ALL} | █ Total\n")
    return buffer.getvalue()


def partition_list(
    data: list[int],
    *,
    partition_size: int = 500,
    min_value: int | None = 0,
    max_value: int | None = None,
) -> list[tuple[str, int]]:
    """
    Partition the list into tuples of the form (range, count).
    The ranges are saved as a string: "0-499", "500-999", etc.
    If no max_value is given, the maximum value in the data is used.
    If min_value is None, the minimum value in the data is used. Otherwise it defaults to 0.

    :param data: The data to partition.
    :param partition_size: [Optional] The size of the partitions. Default: 500.
    :param min_value: [Optional] The minimum value to partition. Default: 0.
    :param max_value: [Optional] The maximum value to partition to. Default: None.
    :return: The partitioned data as a list of tuples.
    """
    if max_value is None:
        max_value = max(data)

    max_label_len = len(str(max_value))

    partition = [
        (
            f"{i * partition_size: >{max_label_len}}-{(i + 1) * partition_size: >{max_label_len}}",
            len([d for d in data if i * partition_size <= d < (i + 1) * partition_size]),
        )
        for i in range(min_value, max_value // partition_size)
    ]

    return partition


def show_masker_analytics(analytics: list[st.MaskFittingAnalytic], max_columns: int = 100) -> str:
    """
    Present the analytics gathered from the masking process.

    :param analytics: The analytics gathered from each page.
    :param max_columns: The maximum number of columns to use for the chart per line.
    :return: The analytics as a string.
    """
    buffer = StringIO()

    total_boxes = len(analytics)
    masks_succeeded = sum(1 for analytic in analytics if analytic.fit_was_found)
    perfect_masks = sum(
        1 for analytic in analytics if analytic.fit_was_found and analytic.mask_std_deviation == 0
    )
    average_border_deviation = (
        f"{sum(analytic.mask_std_deviation for analytic in analytics if analytic.fit_was_found) / masks_succeeded:.2f}"
        if masks_succeeded
        else "N/A"
    )
    success_rate = f"{masks_succeeded / total_boxes:.0%}" if total_boxes else "N/A"
    perfect_mask_rate = f"{perfect_masks / masks_succeeded:.0%}" if masks_succeeded else "N/A"
    masks_failed = total_boxes - masks_succeeded

    highest_mask_index = max(
        (analytic.mask_index for analytic in analytics if analytic.fit_was_found), default=0
    )
    # Count the number of times each mask index was used.
    mask_usages_by_index = [0] * (highest_mask_index + 1)
    # Count the number of times each mask was perfect.
    perfect_mask_usages_by_index = [0] * (highest_mask_index + 1)
    for analytic in analytics:
        if analytic.fit_was_found:
            mask_usages_by_index[analytic.mask_index] += 1
            if analytic.mask_std_deviation == 0:
                perfect_mask_usages_by_index[analytic.mask_index] += 1

    # Write the analytics.
    buffer.write("\nMask Fitment Analytics\n")
    buffer.write("----------------------\n")
    buffer.write(
        f"Total boxes: {total_boxes} | "
        f"Masks succeeded: {masks_succeeded} ({success_rate}) | "
        f"Masks failed: {clr.Fore.RED}{masks_failed}{clr.Fore.RESET}\n"
    )
    buffer.write(
        f"Perfect masks: {clr.Fore.CYAN}{perfect_masks}{clr.Fore.RESET} ({perfect_mask_rate}) | "
        f"Average border deviation: {average_border_deviation}\n"
    )
    buffer.write("\nMask usage by mask size (smallest to largest):\n")
    mask_usages_dict = {
        f"Mask {index}": perfect_total
        for index, perfect_total in enumerate(
            zip(perfect_mask_usages_by_index, mask_usages_by_index)
        )
        if index < len(mask_usages_by_index) - 1
    }
    mask_usages_dict["Box mask"] = perfect_mask_usages_by_index[-1], mask_usages_by_index[-1]

    buffer.write(draw_masker_histogram(mask_usages_dict, max_columns) + "\n")

    pages_with_success_and_fails_dict: defaultdict[Path, defaultdict[str, int]] = defaultdict(
        lambda: defaultdict(int)
    )
    for analytic in analytics:
        pages_with_success_and_fails_dict[analytic.image_path][
            "succeeded" if analytic.fit_was_found else "failed"
        ] += 1

    if not pages_with_success_and_fails_dict:
        buffer.write("All bubbles were successfully masked.\n")
        return buffer.getvalue()

    pages_with_success_and_fails: list[tuple[Path, int, int]] = [
        (page_path, counts["succeeded"], counts["failed"])
        for page_path, counts in pages_with_success_and_fails_dict.items()
        if counts["failed"]
    ]

    if pages_with_success_and_fails:
        page_paths, succeeded_counts, failed_counts = zip(*pages_with_success_and_fails)
        page_paths = hp.trim_prefix_from_paths(page_paths)
        pages_with_success_and_fails = list(zip(page_paths, succeeded_counts, failed_counts))
        pages_with_success_and_fails = natsorted(pages_with_success_and_fails, key=lambda x: x[0])

    buffer.write("\nPages with failures / total:\n")
    for page_path, succeeded, failed in pages_with_success_and_fails:
        buffer.write(f"{page_path}: {clr.Fore.RED}{failed}{clr.Fore.RESET} / {succeeded+failed}\n")

    buffer.write("\n")

    return buffer.getvalue()


def show_denoise_analytics(
    analytics: list[st.DenoiseAnalytic],
    configured_deviation_threshold: float,
    max_deviation_threshold: float,
    max_columns: int = 100,
) -> str:
    """
    Present the analytics gathered from the denoising process.

    :param analytics: The analytics gathered from each page, here standard deviations.
    :param configured_deviation_threshold: The configured deviation threshold to de denoising.
    :param max_deviation_threshold: The maximum deviation threshold to discard the mask.
    :param max_columns: The maximum number of columns to use for the chart per line.
    :return: The analytics as a string.
    """
    buffer = StringIO()

    buffer.write("\nDenoising Analytics\n")
    buffer.write("-------------------\n")

    # Get the standard deviations into one single list.
    std_deviations: list[float] = [
        *itertools.chain.from_iterable(analytic.std_deviations for analytic in analytics)
    ]

    # Split the standard deviations into two lists: one for the ones that were below the threshold,
    # and one for the ones that were above the threshold.
    below_threshold = [
        std_dev for std_dev in std_deviations if std_dev <= configured_deviation_threshold
    ]
    above_threshold = [
        std_dev for std_dev in std_deviations if std_dev > configured_deviation_threshold
    ]

    total_masks = len(std_deviations)
    above_threshold_count = len(above_threshold)
    above_ratio = f"{above_threshold_count / total_masks:.0%}" if total_masks else "N/A"

    buffer.write(
        f"Total masks: {total_masks} | Masks denoised: {clr.Fore.MAGENTA}{above_threshold_count}{clr.Fore.RESET} "
        f"({above_ratio})\n"
    )
    buffer.write(
        f"Minimum deviation to denoise: {configured_deviation_threshold} | "
        f"Maximum allowed deviation: {max_deviation_threshold}\n"
    )
    buffer.write(f"Standard deviation around masks:\n\n")

    # Show the lists in a graph.
    buffer.write(
        draw_denoise_histogram(
            below_threshold,
            above_threshold,
            max_deviation_threshold,
            max_columns,
        )
        + "\n"
    )
    buffer.write("\n")

    return buffer.getvalue()


def draw_denoise_histogram(
    below_threshold: list[float],
    above_threshold: list[float],
    max_value: float,
    max_columns: int = 100,
) -> str:
    """
    Draw a histogram of the deviations.

    :param below_threshold: The standard deviations that were below the threshold.
    :param above_threshold: The standard deviations that were above the threshold.
    :param max_value: The maximum deviation threshold to discard the mask.
    :param max_columns: The maximum number of columns to use for the histogram per line.
    :return: The histogram as a string.
    """
    # Partition the values into 10 buckets.
    # Use a cubic polynomial that passes through 0 and the max value.
    # The polynomial is a simple f(x) = ax^3, meaning the coefficient a
    # must be max_value/10^3.
    exponent = 3
    cubic_coefficient = max_value / 10**exponent
    # The buckets contain: (min, max)
    bucket_ranges = (
        (cubic_coefficient * (i - 1) ** exponent, cubic_coefficient * i**exponent)
        for i in range(1, 11)
    )
    # Set the precision so that the second bucket isn't 0.
    # Basically, if the value is 0.00143, we want a precision of 3 to still see that 1.
    second_bucket_min = str(cubic_coefficient * 1**exponent).split(".")[1]
    # Count the zeroes from the left, and add 1 to only show up to the first non-zero digit.
    precision = len(second_bucket_min) - len(second_bucket_min.lstrip("0")) + 1
    # The buckets contain: (min, max, count_below_threshold, count_above_threshold)
    buckets: list[tuple[str, str, int, int]] = [
        (
            f"{b_min:.{precision}f}",
            f"{b_max:.{precision}f}",
            sum(1 for std_dev in below_threshold if b_min <= std_dev < b_max),
            sum(1 for std_dev in above_threshold if b_min <= std_dev < b_max),
        )
        for b_min, b_max in bucket_ranges
    ]
    # Format the ranges to look better.
    number_width = len(str(f"{max_value:.{precision}f}"))
    buckets_labeled: list[tuple[str, int, int]] = [
        (
            f"{b_min: >{number_width}}-{b_max: >{number_width}}",
            below,
            above,
        )
        for b_min, b_max, below, above in buckets
    ]
    # Get the maximum count in the buckets to rescale them.
    max_count = max((below + above for _, below, above in buckets_labeled))
    max_count = max(max_count, 1)
    # Rescale the buckets.

    max_label_length = max(len(label) for label, _, _ in buckets_labeled)
    bar_width = max_columns - max_label_length - 16  # 16 is for the spaces and the brackets.

    bucket_bar_lengths: list[tuple[int, int]] = [
        (int(below * bar_width / max_count), int(above * bar_width / max_count))
        for _, below, above in buckets_labeled
    ]

    buffer = StringIO()
    # Draw the buckets.
    for (b_range, below, above), (below_len, above_len) in zip(buckets_labeled, bucket_bar_lengths):
        buffer.write(
            f"{b_range} : {'█' * below_len}{clr.Fore.MAGENTA}{'█' * above_len} {above}{clr.Fore.RESET} / {below+above}\n"
        )

    # Draw the legend.
    buffer.write(f"\n{clr.Fore.MAGENTA}█ Denoised{clr.Fore.RESET} | █ Total\n")
    return buffer.getvalue()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'Mainwindow.ui'
##
## Created by: Qt User Interface Compiler version 6.5.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QCheckBox, QFrame,
    QGroupBox, QHBoxLayout, QHeaderView, QLabel,
    QLineEdit, QMainWindow, QMenu, QMenuBar,
    QProgressBar, QPushButton, QRadioButton, QSizePolicy,
    QSpacerItem, QSplitter, QStackedWidget, QStatusBar,
    QTableWidgetItem, QTextEdit, QVBoxLayout, QWidget)

from pcleaner.gui.CustomQ.CComboBox import CComboBox
from pcleaner.gui.CustomQ.CDropFrame import CDropFrame
from pcleaner.gui.CustomQ.CTooltipLabel import CTooltipLabel
from pcleaner.gui.file_table import FileTable
from pcleaner.gui.image_tab import ImageTab

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1800, 720)
        self.action_add_files = QAction(MainWindow)
        self.action_add_files.setObjectName(u"action_add_files")
        icon = QIcon()
        iconThemeName = u"document-open"
        if QIcon.hasThemeIcon(iconThemeName):
            icon = QIcon.fromTheme(iconThemeName)
        else:
            icon.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_add_files.setIcon(icon)
        self.action_clear_files = QAction(MainWindow)
        self.action_clear_files.setObjectName(u"action_clear_files")
        icon1 = QIcon()
        iconThemeName = u"edit-clear-history"
        if QIcon.hasThemeIcon(iconThemeName):
            icon1 = QIcon.fromTheme(iconThemeName)
        else:
            icon1.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_clear_files.setIcon(icon1)
        self.action_new_profile = QAction(MainWindow)
        self.action_new_profile.setObjectName(u"action_new_profile")
        icon2 = QIcon()
        iconThemeName = u"document-new"
        if QIcon.hasThemeIcon(iconThemeName):
            icon2 = QIcon.fromTheme(iconThemeName)
        else:
            icon2.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_new_profile.setIcon(icon2)
        self.action_delete_profile = QAction(MainWindow)
        self.action_delete_profile.setObjectName(u"action_delete_profile")
        icon3 = QIcon()
        iconThemeName = u"edit-delete"
        if QIcon.hasThemeIcon(iconThemeName):
            icon3 = QIcon.fromTheme(iconThemeName)
        else:
            icon3.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_delete_profile.setIcon(icon3)
        self.action_import_profile = QAction(MainWindow)
        self.action_import_profile.setObjectName(u"action_import_profile")
        icon4 = QIcon()
        iconThemeName = u"document-import"
        if QIcon.hasThemeIcon(iconThemeName):
            icon4 = QIcon.fromTheme(iconThemeName)
        else:
            icon4.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_import_profile.setIcon(icon4)
        self.action_open_log = QAction(MainWindow)
        self.action_open_log.setObjectName(u"action_open_log")
        self.action_online_documentation = QAction(MainWindow)
        self.action_online_documentation.setObjectName(u"action_online_documentation")
        icon5 = QIcon()
        iconThemeName = u"internet-services"
        if QIcon.hasThemeIcon(iconThemeName):
            icon5 = QIcon.fromTheme(iconThemeName)
        else:
            icon5.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_online_documentation.setIcon(icon5)
        self.action_about = QAction(MainWindow)
        self.action_about.setObjectName(u"action_about")
        icon6 = QIcon()
        iconThemeName = u"help-about"
        if QIcon.hasThemeIcon(iconThemeName):
            icon6 = QIcon.fromTheme(iconThemeName)
        else:
            icon6.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_about.setIcon(icon6)
        self.actionView_License = QAction(MainWindow)
        self.actionView_License.setObjectName(u"actionView_License")
        self.action_save_profile = QAction(MainWindow)
        self.action_save_profile.setObjectName(u"action_save_profile")
        icon7 = QIcon()
        iconThemeName = u"document-save"
        if QIcon.hasThemeIcon(iconThemeName):
            icon7 = QIcon.fromTheme(iconThemeName)
        else:
            icon7.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_save_profile.setIcon(icon7)
        self.action_save_profile_as = QAction(MainWindow)
        self.action_save_profile_as.setObjectName(u"action_save_profile_as")
        icon8 = QIcon()
        iconThemeName = u"document-save-as"
        if QIcon.hasThemeIcon(iconThemeName):
            icon8 = QIcon.fromTheme(iconThemeName)
        else:
            icon8.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_save_profile_as.setIcon(icon8)
        self.action_Apply_Denoising = QAction(MainWindow)
        self.action_Apply_Denoising.setObjectName(u"action_Apply_Denoising")
        self.action_Apply_Denoising.setCheckable(True)
        self.action_Apply_Denoising.setChecked(True)
        self.action_show_terminal_command = QAction(MainWindow)
        self.action_show_terminal_command.setObjectName(u"action_show_terminal_command")
        self.action_system_theme = QAction(MainWindow)
        self.action_system_theme.setObjectName(u"action_system_theme")
        self.action_system_theme.setCheckable(True)
        self.action_dark = QAction(MainWindow)
        self.action_dark.setObjectName(u"action_dark")
        self.action_dark.setCheckable(True)
        self.action_light = QAction(MainWindow)
        self.action_light.setObjectName(u"action_light")
        self.action_light.setCheckable(True)
        self.action_temp = QAction(MainWindow)
        self.action_temp.setObjectName(u"action_temp")
        self.action_temp_2 = QAction(MainWindow)
        self.action_temp_2.setObjectName(u"action_temp_2")
        self.action_add_folders = QAction(MainWindow)
        self.action_add_folders.setObjectName(u"action_add_folders")
        icon9 = QIcon()
        iconThemeName = u"document-open-folder"
        if QIcon.hasThemeIcon(iconThemeName):
            icon9 = QIcon.fromTheme(iconThemeName)
        else:
            icon9.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_add_folders.setIcon(icon9)
        self.action_delete_models = QAction(MainWindow)
        self.action_delete_models.setObjectName(u"action_delete_models")
        self.action_delete_models.setIcon(icon3)
        self.action_download_models = QAction(MainWindow)
        self.action_download_models.setObjectName(u"action_download_models")
        icon10 = QIcon()
        iconThemeName = u"download"
        if QIcon.hasThemeIcon(iconThemeName):
            icon10 = QIcon.fromTheme(iconThemeName)
        else:
            icon10.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.action_download_models.setIcon(icon10)
        self.action_donate = QAction(MainWindow)
        self.action_donate.setObjectName(u"action_donate")
        icon11 = QIcon()
        icon11.addFile(u":/heart.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.action_donate.setIcon(icon11)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout_6 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.verticalLayout_6.setContentsMargins(1, 4, 1, 1)
        self.splitter = QSplitter(self.centralwidget)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.splitter.setHandleWidth(10)
        self.groupBox_profile = QGroupBox(self.splitter)
        self.groupBox_profile.setObjectName(u"groupBox_profile")
        self.verticalLayout_2 = QVBoxLayout(self.groupBox_profile)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_profile_header = QHBoxLayout()
        self.horizontalLayout_profile_header.setObjectName(u"horizontalLayout_profile_header")
        self.comboBox_current_profile = CComboBox(self.groupBox_profile)
        self.comboBox_current_profile.addItem("")
        self.comboBox_current_profile.setObjectName(u"comboBox_current_profile")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox_current_profile.sizePolicy().hasHeightForWidth())
        self.comboBox_current_profile.setSizePolicy(sizePolicy)
        self.comboBox_current_profile.setMinimumSize(QSize(48, 0))

        self.horizontalLayout_profile_header.addWidget(self.comboBox_current_profile)

        self.pushButton_apply_profile = QPushButton(self.groupBox_profile)
        self.pushButton_apply_profile.setObjectName(u"pushButton_apply_profile")
        self.pushButton_apply_profile.setEnabled(False)
        icon12 = QIcon()
        iconThemeName = u"dialog-ok-apply"
        if QIcon.hasThemeIcon(iconThemeName):
            icon12 = QIcon.fromTheme(iconThemeName)
        else:
            icon12.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_apply_profile.setIcon(icon12)

        self.horizontalLayout_profile_header.addWidget(self.pushButton_apply_profile)

        self.pushButton_save_profile = QPushButton(self.groupBox_profile)
        self.pushButton_save_profile.setObjectName(u"pushButton_save_profile")
        self.pushButton_save_profile.setEnabled(False)
        self.pushButton_save_profile.setIcon(icon7)

        self.horizontalLayout_profile_header.addWidget(self.pushButton_save_profile)

        self.pushButton_reset_profile = QPushButton(self.groupBox_profile)
        self.pushButton_reset_profile.setObjectName(u"pushButton_reset_profile")
        self.pushButton_reset_profile.setEnabled(False)
        icon13 = QIcon()
        iconThemeName = u"document-revert"
        if QIcon.hasThemeIcon(iconThemeName):
            icon13 = QIcon.fromTheme(iconThemeName)
        else:
            icon13.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_reset_profile.setIcon(icon13)

        self.horizontalLayout_profile_header.addWidget(self.pushButton_reset_profile)


        self.verticalLayout_2.addLayout(self.horizontalLayout_profile_header)

        self.toolBox_profile_frame = QWidget(self.groupBox_profile)
        self.toolBox_profile_frame.setObjectName(u"toolBox_profile_frame")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.toolBox_profile_frame.sizePolicy().hasHeightForWidth())
        self.toolBox_profile_frame.setSizePolicy(sizePolicy1)

        self.verticalLayout_2.addWidget(self.toolBox_profile_frame)

        self.splitter.addWidget(self.groupBox_profile)
        self.stackedWidget_images = QStackedWidget(self.splitter)
        self.stackedWidget_images.setObjectName(u"stackedWidget_images")
        self.page_greeter = QWidget()
        self.page_greeter.setObjectName(u"page_greeter")
        self.verticalLayout_8 = QVBoxLayout(self.page_greeter)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.frame_greeter = CDropFrame(self.page_greeter)
        self.frame_greeter.setObjectName(u"frame_greeter")
        self.frame_greeter.setFrameShape(QFrame.StyledPanel)
        self.frame_greeter.setFrameShadow(QFrame.Raised)
        self.frame_greeter.setLineWidth(4)
        self.horizontalLayout_4 = QHBoxLayout(self.frame_greeter)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.verticalLayout_10 = QVBoxLayout()
        self.verticalLayout_10.setSpacing(30)
        self.verticalLayout_10.setObjectName(u"verticalLayout_10")
        self.verticalSpacer_3 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_10.addItem(self.verticalSpacer_3)

        self.label_drop = QLabel(self.frame_greeter)
        self.label_drop.setObjectName(u"label_drop")
        self.label_drop.setAlignment(Qt.AlignCenter)

        self.verticalLayout_10.addWidget(self.label_drop)

        self.label_drop_icon = QLabel(self.frame_greeter)
        self.label_drop_icon.setObjectName(u"label_drop_icon")
        self.label_drop_icon.setAlignment(Qt.AlignCenter)

        self.verticalLayout_10.addWidget(self.label_drop_icon)

        self.verticalSpacer_4 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_10.addItem(self.verticalSpacer_4)


        self.horizontalLayout_4.addLayout(self.verticalLayout_10)


        self.verticalLayout_8.addWidget(self.frame_greeter)

        self.stackedWidget_images.addWidget(self.page_greeter)
        self.page_table = QWidget()
        self.page_table.setObjectName(u"page_table")
        self.verticalLayout = QVBoxLayout(self.page_table)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.image_tab = ImageTab(self.page_table)
        self.image_tab.setObjectName(u"image_tab")
        self.image_tab.setTabsClosable(True)
        self.tabWidget_table_page = QWidget()
        self.tabWidget_table_page.setObjectName(u"tabWidget_table_page")
        self.verticalLayout_3 = QVBoxLayout(self.tabWidget_table_page)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.file_table = FileTable(self.tabWidget_table_page)
        if (self.file_table.columnCount() < 7):
            self.file_table.setColumnCount(7)
        __qtablewidgetitem = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        __qtablewidgetitem2 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(2, __qtablewidgetitem2)
        __qtablewidgetitem3 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(3, __qtablewidgetitem3)
        __qtablewidgetitem4 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(4, __qtablewidgetitem4)
        __qtablewidgetitem5 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(5, __qtablewidgetitem5)
        __qtablewidgetitem6 = QTableWidgetItem()
        self.file_table.setHorizontalHeaderItem(6, __qtablewidgetitem6)
        self.file_table.setObjectName(u"file_table")
        self.file_table.setFrameShape(QFrame.NoFrame)
        self.file_table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.file_table.setDragDropOverwriteMode(False)
        self.file_table.setDragDropMode(QAbstractItemView.NoDragDrop)
        self.file_table.setAlternatingRowColors(True)
        self.file_table.setSelectionMode(QAbstractItemView.SingleSelection)
        self.file_table.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.file_table.setSortingEnabled(False)
        self.file_table.setCornerButtonEnabled(False)
        self.file_table.horizontalHeader().setHighlightSections(False)
        self.file_table.verticalHeader().setHighlightSections(False)

        self.verticalLayout_3.addWidget(self.file_table)

        self.image_tab.addTab(self.tabWidget_table_page, "")

        self.verticalLayout.addWidget(self.image_tab)

        self.stackedWidget_images.addWidget(self.page_table)
        self.splitter.addWidget(self.stackedWidget_images)
        self.frame_output = QFrame(self.splitter)
        self.frame_output.setObjectName(u"frame_output")
        self.frame_output.setFrameShape(QFrame.NoFrame)
        self.frame_output.setFrameShadow(QFrame.Raised)
        self.verticalLayout_4 = QVBoxLayout(self.frame_output)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.groupBox_4 = QGroupBox(self.frame_output)
        self.groupBox_4.setObjectName(u"groupBox_4")
        self.verticalLayout_9 = QVBoxLayout(self.groupBox_4)
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.radioButton_cleaning = QRadioButton(self.groupBox_4)
        self.radioButton_cleaning.setObjectName(u"radioButton_cleaning")
        self.radioButton_cleaning.setChecked(True)

        self.verticalLayout_9.addWidget(self.radioButton_cleaning)

        self.radioButton_ocr = QRadioButton(self.groupBox_4)
        self.radioButton_ocr.setObjectName(u"radioButton_ocr")

        self.verticalLayout_9.addWidget(self.radioButton_ocr)

        self.verticalSpacer = QSpacerItem(20, 0, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_9.addItem(self.verticalSpacer)

        self.verticalLayout_7 = QVBoxLayout()
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.label_warning = QLabel(self.groupBox_4)
        self.label_warning.setObjectName(u"label_warning")
        sizePolicy2 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.label_warning.sizePolicy().hasHeightForWidth())
        self.label_warning.setSizePolicy(sizePolicy2)

        self.verticalLayout_7.addWidget(self.label_warning)

        self.pushButton_abort = QPushButton(self.groupBox_4)
        self.pushButton_abort.setObjectName(u"pushButton_abort")
        icon14 = QIcon()
        iconThemeName = u"process-stop"
        if QIcon.hasThemeIcon(iconThemeName):
            icon14 = QIcon.fromTheme(iconThemeName)
        else:
            icon14.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_abort.setIcon(icon14)

        self.verticalLayout_7.addWidget(self.pushButton_abort)

        self.pushButton_start = QPushButton(self.groupBox_4)
        self.pushButton_start.setObjectName(u"pushButton_start")
        icon15 = QIcon()
        iconThemeName = u"media-playback-start"
        if QIcon.hasThemeIcon(iconThemeName):
            icon15 = QIcon.fromTheme(iconThemeName)
        else:
            icon15.addFile(u".", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_start.setIcon(icon15)

        self.verticalLayout_7.addWidget(self.pushButton_start)


        self.verticalLayout_9.addLayout(self.verticalLayout_7)


        self.horizontalLayout_3.addWidget(self.groupBox_4)

        self.groupBox_3 = QGroupBox(self.frame_output)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.verticalLayout_5 = QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.stackedWidget_output = QStackedWidget(self.groupBox_3)
        self.stackedWidget_output.setObjectName(u"stackedWidget_output")
        self.page_cleaning = QWidget()
        self.page_cleaning.setObjectName(u"page_cleaning")
        self.verticalLayout_14 = QVBoxLayout(self.page_cleaning)
        self.verticalLayout_14.setObjectName(u"verticalLayout_14")
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.verticalLayout_11 = QVBoxLayout()
        self.verticalLayout_11.setSpacing(6)
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.checkBox_save_clean = QCheckBox(self.page_cleaning)
        self.checkBox_save_clean.setObjectName(u"checkBox_save_clean")
        self.checkBox_save_clean.setChecked(True)

        self.verticalLayout_11.addWidget(self.checkBox_save_clean)

        self.checkBox_save_mask = QCheckBox(self.page_cleaning)
        self.checkBox_save_mask.setObjectName(u"checkBox_save_mask")
        self.checkBox_save_mask.setChecked(True)

        self.verticalLayout_11.addWidget(self.checkBox_save_mask)

        self.checkBox_save_text = QCheckBox(self.page_cleaning)
        self.checkBox_save_text.setObjectName(u"checkBox_save_text")

        self.verticalLayout_11.addWidget(self.checkBox_save_text)


        self.horizontalLayout_7.addLayout(self.verticalLayout_11)

        self.horizontalSpacer_6 = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_7.addItem(self.horizontalSpacer_6)

        self.verticalLayout_13 = QVBoxLayout()
        self.verticalLayout_13.setSpacing(6)
        self.verticalLayout_13.setObjectName(u"verticalLayout_13")
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.checkBox_write_output = QCheckBox(self.page_cleaning)
        self.checkBox_write_output.setObjectName(u"checkBox_write_output")
        self.checkBox_write_output.setChecked(True)

        self.horizontalLayout_6.addWidget(self.checkBox_write_output)

        self.label_write_output_help = CTooltipLabel(self.page_cleaning)
        self.label_write_output_help.setObjectName(u"label_write_output_help")
        self.label_write_output_help.setToolTipDuration(-1)

        self.horizontalLayout_6.addWidget(self.label_write_output_help)

        self.horizontalSpacer_4 = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_6.addItem(self.horizontalSpacer_4)


        self.verticalLayout_13.addLayout(self.horizontalLayout_6)

        self.verticalSpacer_6 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_13.addItem(self.verticalSpacer_6)


        self.horizontalLayout_7.addLayout(self.verticalLayout_13)


        self.verticalLayout_14.addLayout(self.horizontalLayout_7)

        self.verticalSpacer_2 = QSpacerItem(20, 1, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_14.addItem(self.verticalSpacer_2)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label_4 = QLabel(self.page_cleaning)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setTextInteractionFlags(Qt.LinksAccessibleByMouse)

        self.horizontalLayout_5.addWidget(self.label_4)

        self.label_cleaning_outdir_help = CTooltipLabel(self.page_cleaning)
        self.label_cleaning_outdir_help.setObjectName(u"label_cleaning_outdir_help")
        self.label_cleaning_outdir_help.setToolTipDuration(-1)

        self.horizontalLayout_5.addWidget(self.label_cleaning_outdir_help)

        self.horizontalSpacer_3 = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_5.addItem(self.horizontalSpacer_3)


        self.verticalLayout_14.addLayout(self.horizontalLayout_5)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.lineEdit_out_directory = QLineEdit(self.page_cleaning)
        self.lineEdit_out_directory.setObjectName(u"lineEdit_out_directory")
        self.lineEdit_out_directory.setClearButtonEnabled(True)

        self.horizontalLayout.addWidget(self.lineEdit_out_directory)

        self.pushButton_browse_out_dir = QPushButton(self.page_cleaning)
        self.pushButton_browse_out_dir.setObjectName(u"pushButton_browse_out_dir")
        self.pushButton_browse_out_dir.setIcon(icon9)

        self.horizontalLayout.addWidget(self.pushButton_browse_out_dir)


        self.verticalLayout_14.addLayout(self.horizontalLayout)

        self.stackedWidget_output.addWidget(self.page_cleaning)
        self.page_ocr = QWidget()
        self.page_ocr.setObjectName(u"page_ocr")
        self.verticalLayout_12 = QVBoxLayout(self.page_ocr)
        self.verticalLayout_12.setObjectName(u"verticalLayout_12")
        self.verticalLayout_12.setContentsMargins(0, 0, 0, 0)
        self.radioButton_ocr_text = QRadioButton(self.page_ocr)
        self.radioButton_ocr_text.setObjectName(u"radioButton_ocr_text")
        self.radioButton_ocr_text.setChecked(True)

        self.verticalLayout_12.addWidget(self.radioButton_ocr_text)

        self.radioButton_ocr_csv = QRadioButton(self.page_ocr)
        self.radioButton_ocr_csv.setObjectName(u"radioButton_ocr_csv")

        self.verticalLayout_12.addWidget(self.radioButton_ocr_csv)

        self.verticalSpacer_5 = QSpacerItem(10, 37, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_12.addItem(self.verticalSpacer_5)

        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.label_6 = QLabel(self.page_ocr)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setTextInteractionFlags(Qt.LinksAccessibleByMouse)

        self.horizontalLayout_9.addWidget(self.label_6)

        self.label_ocr_outdir_help = CTooltipLabel(self.page_ocr)
        self.label_ocr_outdir_help.setObjectName(u"label_ocr_outdir_help")
        self.label_ocr_outdir_help.setToolTipDuration(-1)

        self.horizontalLayout_9.addWidget(self.label_ocr_outdir_help)

        self.horizontalSpacer_5 = QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_9.addItem(self.horizontalSpacer_5)


        self.verticalLayout_12.addLayout(self.horizontalLayout_9)

        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.lineEdit_out_file = QLineEdit(self.page_ocr)
        self.lineEdit_out_file.setObjectName(u"lineEdit_out_file")
        self.lineEdit_out_file.setClearButtonEnabled(True)

        self.horizontalLayout_8.addWidget(self.lineEdit_out_file)

        self.pushButton_browse_out_file = QPushButton(self.page_ocr)
        self.pushButton_browse_out_file.setObjectName(u"pushButton_browse_out_file")
        self.pushButton_browse_out_file.setIcon(icon)

        self.horizontalLayout_8.addWidget(self.pushButton_browse_out_file)


        self.verticalLayout_12.addLayout(self.horizontalLayout_8)

        self.stackedWidget_output.addWidget(self.page_ocr)

        self.verticalLayout_5.addWidget(self.stackedWidget_output)


        self.horizontalLayout_3.addWidget(self.groupBox_3)

        self.horizontalLayout_3.setStretch(0, 1)
        self.horizontalLayout_3.setStretch(1, 3)

        self.verticalLayout_4.addLayout(self.horizontalLayout_3)

        self.textEdit_analytics = QTextEdit(self.frame_output)
        self.textEdit_analytics.setObjectName(u"textEdit_analytics")
        self.textEdit_analytics.setUndoRedoEnabled(False)
        self.textEdit_analytics.setLineWrapMode(QTextEdit.NoWrap)
        self.textEdit_analytics.setReadOnly(True)

        self.verticalLayout_4.addWidget(self.textEdit_analytics)

        self.verticalLayout_4.setStretch(1, 1)
        self.splitter.addWidget(self.frame_output)

        self.verticalLayout_6.addWidget(self.splitter)

        self.widget_progress_drawer = QWidget(self.centralwidget)
        self.widget_progress_drawer.setObjectName(u"widget_progress_drawer")
        self.horizontalLayout_2 = QHBoxLayout(self.widget_progress_drawer)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.label_generating_what = QLabel(self.widget_progress_drawer)
        self.label_generating_what.setObjectName(u"label_generating_what")

        self.horizontalLayout_2.addWidget(self.label_generating_what)

        self.label_target_outputs = QLabel(self.widget_progress_drawer)
        self.label_target_outputs.setObjectName(u"label_target_outputs")

        self.horizontalLayout_2.addWidget(self.label_target_outputs)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Maximum, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.label_progress_total = QLabel(self.widget_progress_drawer)
        self.label_progress_total.setObjectName(u"label_progress_total")

        self.horizontalLayout_2.addWidget(self.label_progress_total)

        self.progressBar_total = QProgressBar(self.widget_progress_drawer)
        self.progressBar_total.setObjectName(u"progressBar_total")
        sizePolicy3 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.progressBar_total.sizePolicy().hasHeightForWidth())
        self.progressBar_total.setSizePolicy(sizePolicy3)
        self.progressBar_total.setMinimum(0)
        self.progressBar_total.setValue(24)
        self.progressBar_total.setTextVisible(True)

        self.horizontalLayout_2.addWidget(self.progressBar_total)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Maximum, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_2)

        self.label_progress_step = QLabel(self.widget_progress_drawer)
        self.label_progress_step.setObjectName(u"label_progress_step")

        self.horizontalLayout_2.addWidget(self.label_progress_step)

        self.progressBar_individual = QProgressBar(self.widget_progress_drawer)
        self.progressBar_individual.setObjectName(u"progressBar_individual")
        self.progressBar_individual.setValue(24)

        self.horizontalLayout_2.addWidget(self.progressBar_individual)

        self.horizontalLayout_2.setStretch(4, 1)
        self.horizontalLayout_2.setStretch(7, 5)

        self.verticalLayout_6.addWidget(self.widget_progress_drawer)

        self.verticalLayout_6.setStretch(0, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1800, 30))
        self.menu_File = QMenu(self.menubar)
        self.menu_File.setObjectName(u"menu_File")
        self.menu_Profile = QMenu(self.menubar)
        self.menu_Profile.setObjectName(u"menu_Profile")
        self.menu_set_default_profile = QMenu(self.menu_Profile)
        self.menu_set_default_profile.setObjectName(u"menu_set_default_profile")
        self.menu_settings = QMenu(self.menubar)
        self.menu_settings.setObjectName(u"menu_settings")
        self.menu_theme = QMenu(self.menu_settings)
        self.menu_theme.setObjectName(u"menu_theme")
        self.menu_Help = QMenu(self.menubar)
        self.menu_Help.setObjectName(u"menu_Help")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
#if QT_CONFIG(shortcut)
        self.label_4.setBuddy(self.lineEdit_out_directory)
        self.label_6.setBuddy(self.lineEdit_out_directory)
#endif // QT_CONFIG(shortcut)

        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Profile.menuAction())
        self.menubar.addAction(self.menu_settings.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.menu_File.addAction(self.action_add_files)
        self.menu_File.addAction(self.action_add_folders)
        self.menu_File.addAction(self.action_clear_files)
        self.menu_Profile.addAction(self.action_new_profile)
        self.menu_Profile.addAction(self.action_import_profile)
        self.menu_Profile.addSeparator()
        self.menu_Profile.addAction(self.action_save_profile)
        self.menu_Profile.addAction(self.action_save_profile_as)
        self.menu_Profile.addSeparator()
        self.menu_Profile.addAction(self.action_delete_profile)
        self.menu_Profile.addSeparator()
        self.menu_Profile.addAction(self.menu_set_default_profile.menuAction())
        self.menu_set_default_profile.addAction(self.action_temp_2)
        self.menu_settings.addAction(self.menu_theme.menuAction())
        self.menu_theme.addAction(self.action_system_theme)
        self.menu_theme.addSeparator()
        self.menu_theme.addAction(self.action_dark)
        self.menu_theme.addAction(self.action_light)
        self.menu_Help.addAction(self.action_online_documentation)
        self.menu_Help.addAction(self.action_about)
        self.menu_Help.addAction(self.action_donate)
        self.menu_Help.addSeparator()
        self.menu_Help.addAction(self.action_delete_models)

        self.retranslateUi(MainWindow)

        self.stackedWidget_images.setCurrentIndex(0)
        self.stackedWidget_output.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"Panel Cleaner", None))
        self.action_add_files.setText(QCoreApplication.translate("MainWindow", u"Add Files...", None))
        self.action_clear_files.setText(QCoreApplication.translate("MainWindow", u"Clear Files", None))
        self.action_new_profile.setText(QCoreApplication.translate("MainWindow", u"New", None))
        self.action_delete_profile.setText(QCoreApplication.translate("MainWindow", u"Delete", None))
        self.action_import_profile.setText(QCoreApplication.translate("MainWindow", u"Import...", None))
        self.action_open_log.setText(QCoreApplication.translate("MainWindow", u"Open Log", None))
        self.action_online_documentation.setText(QCoreApplication.translate("MainWindow", u"Online Documentation", None))
        self.action_about.setText(QCoreApplication.translate("MainWindow", u"About Panel Cleaner", None))
        self.actionView_License.setText(QCoreApplication.translate("MainWindow", u"View License", None))
        self.action_save_profile.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.action_save_profile_as.setText(QCoreApplication.translate("MainWindow", u"Save as...", None))
        self.action_Apply_Denoising.setText(QCoreApplication.translate("MainWindow", u"Apply Denoising", None))
        self.action_show_terminal_command.setText(QCoreApplication.translate("MainWindow", u"Show Terminal Command", None))
        self.action_system_theme.setText(QCoreApplication.translate("MainWindow", u"System", None))
        self.action_dark.setText(QCoreApplication.translate("MainWindow", u"Dark", None))
        self.action_light.setText(QCoreApplication.translate("MainWindow", u"Light", None))
        self.action_temp.setText(QCoreApplication.translate("MainWindow", u"<temp>", None))
        self.action_temp_2.setText(QCoreApplication.translate("MainWindow", u"<temp>", None))
        self.action_add_folders.setText(QCoreApplication.translate("MainWindow", u"Add Folder...", None))
        self.action_delete_models.setText(QCoreApplication.translate("MainWindow", u"Delete Machine Learning Models", None))
        self.action_download_models.setText(QCoreApplication.translate("MainWindow", u"Download Machine Learning Models", None))
        self.action_donate.setText(QCoreApplication.translate("MainWindow", u"Donate", None))
        self.groupBox_profile.setTitle(QCoreApplication.translate("MainWindow", u"Profile", None))
        self.comboBox_current_profile.setItemText(0, QCoreApplication.translate("MainWindow", u"Default", None))

        self.pushButton_apply_profile.setText(QCoreApplication.translate("MainWindow", u"Apply", None))
        self.pushButton_save_profile.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.pushButton_reset_profile.setText(QCoreApplication.translate("MainWindow", u"Reset All", None))
        self.label_drop.setText(QCoreApplication.translate("MainWindow", u"Drag and Drop Images or Folders Here", None))
        self.label_drop_icon.setText(QCoreApplication.translate("MainWindow", u"<drop icon>", None))
        ___qtablewidgetitem = self.file_table.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("MainWindow", u"id", None));
        ___qtablewidgetitem1 = self.file_table.horizontalHeaderItem(1)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("MainWindow", u"File", None));
        ___qtablewidgetitem2 = self.file_table.horizontalHeaderItem(2)
        ___qtablewidgetitem2.setText(QCoreApplication.translate("MainWindow", u"Size", None));
        ___qtablewidgetitem3 = self.file_table.horizontalHeaderItem(3)
        ___qtablewidgetitem3.setText(QCoreApplication.translate("MainWindow", u"Processing Size", None));
        ___qtablewidgetitem4 = self.file_table.horizontalHeaderItem(4)
        ___qtablewidgetitem4.setText(QCoreApplication.translate("MainWindow", u"File Size", None));
        ___qtablewidgetitem5 = self.file_table.horizontalHeaderItem(5)
        ___qtablewidgetitem5.setText(QCoreApplication.translate("MainWindow", u"Color Mode", None));
        ___qtablewidgetitem6 = self.file_table.horizontalHeaderItem(6)
        ___qtablewidgetitem6.setText(QCoreApplication.translate("MainWindow", u"Analytics", None));
        self.image_tab.setTabText(self.image_tab.indexOf(self.tabWidget_table_page), QCoreApplication.translate("MainWindow", u"Images", None))
        self.groupBox_4.setTitle(QCoreApplication.translate("MainWindow", u"Process", None))
        self.radioButton_cleaning.setText(QCoreApplication.translate("MainWindow", u"Cleaning", None))
        self.radioButton_ocr.setText(QCoreApplication.translate("MainWindow", u"OCR", None))
        self.label_warning.setText(QCoreApplication.translate("MainWindow", u"Warning", None))
        self.pushButton_abort.setText(QCoreApplication.translate("MainWindow", u"Abort", None))
        self.pushButton_start.setText(QCoreApplication.translate("MainWindow", u"Start", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("MainWindow", u"Output", None))
        self.checkBox_save_clean.setText(QCoreApplication.translate("MainWindow", u"Cleaned Image", None))
        self.checkBox_save_mask.setText(QCoreApplication.translate("MainWindow", u"Mask", None))
        self.checkBox_save_text.setText(QCoreApplication.translate("MainWindow", u"Isolated Text", None))
        self.checkBox_write_output.setText(QCoreApplication.translate("MainWindow", u"Save Output", None))
#if QT_CONFIG(tooltip)
        self.label_write_output_help.setToolTip(QCoreApplication.translate("MainWindow", u"<html><head/><body><p>When checked, the outputs are saved on disk. Otherwise, you can only preview them in the image details view. </p><p>If the profile remains unchanged after cleaning without this option enabled, you can quickly export them by running the cleaner again with this option enabled.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.label_write_output_help.setText(QCoreApplication.translate("MainWindow", u"<helper>", None))
        self.label_4.setText(QCoreApplication.translate("MainWindow", u"Output Directory:", None))
#if QT_CONFIG(tooltip)
        self.label_cleaning_outdir_help.setToolTip(QCoreApplication.translate("MainWindow", u"You can use a relative path to create a subfolder at the image's original location, or use an absolute path.", None))
#endif // QT_CONFIG(tooltip)
        self.label_cleaning_outdir_help.setText(QCoreApplication.translate("MainWindow", u"<helper>", None))
        self.lineEdit_out_directory.setPlaceholderText(QCoreApplication.translate("MainWindow", u"cleaned", None))
        self.pushButton_browse_out_dir.setText("")
        self.radioButton_ocr_text.setText(QCoreApplication.translate("MainWindow", u"Plain Text", None))
        self.radioButton_ocr_csv.setText(QCoreApplication.translate("MainWindow", u"CSV File", None))
        self.label_6.setText(QCoreApplication.translate("MainWindow", u"Output File:", None))
#if QT_CONFIG(tooltip)
        self.label_ocr_outdir_help.setToolTip(QCoreApplication.translate("MainWindow", u"<html><head/><body><p>Either enter an absolute or relative path with a file name. The output of all images is written to the same file.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.label_ocr_outdir_help.setText(QCoreApplication.translate("MainWindow", u"<helper>", None))
        self.lineEdit_out_file.setPlaceholderText(QCoreApplication.translate("MainWindow", u"detected_text.txt", None))
        self.pushButton_browse_out_file.setText("")
        self.textEdit_analytics.setHtml(QCoreApplication.translate("MainWindow", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Noto Mono','Monospace';\">Mask Fitment Analytics <br />---------------------- <br />Total boxes: 5 | Masks succeeded: 5 (100%) | Masks failed: </span><span style=\" font-family:'Noto Mono','Monospace'; color:#b21818;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> <br />Perfect masks: </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">5</span><span style=\" font-family:'Noto Mono','Monospace';\"> (100%) | Average border deviation: 0.00 <br /><br />Mask usage by mask size (smallest to largest)"
                        ": <br />Mask 0  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 1  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 2  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 3  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 4  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 5  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 6  : </span><span style=\" font-family:'Noto Mono','"
                        "Monospace'; color:#18b2b2;\">\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588</span><span style=\" font-family:'Noto Mono','Monospace';\"> </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">1</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 1 <br />Mask 7  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 8  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 9  :  </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Mask 10 :  </span><span style=\" font-family:'Noto Mono','Monospa"
                        "ce'; color:#18b2b2;\">0</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 0 <br />Box mask: </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588\u2588</span><span style=\" font-family:'Noto Mono','Monospace';\"> </span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">4</span><span style=\" font-family:'Noto Mono','Monospace';\"> / 4 <br /><br /></span><span style=\" font-family:'Noto Mono','Monospace'; color:#18b2b2;\">\u2588 Perfect</span><span style=\" font-family:'Noto Mono','Monospace';\"> | \u2588 Total<br /><br /></span></p></body></html>", None))
        self.label_generating_what.setText(QCoreApplication.translate("MainWindow", u"Generating:", None))
        self.label_target_outputs.setText(QCoreApplication.translate("MainWindow", u"<targeted output(s)>", None))
        self.label_progress_total.setText(QCoreApplication.translate("MainWindow", u"Total Progress:", None))
        self.progressBar_total.setFormat(QCoreApplication.translate("MainWindow", u"%v / %m", None))
        self.label_progress_step.setText(QCoreApplication.translate("MainWindow", u"Images Processed:", None))
        self.progressBar_individual.setFormat(QCoreApplication.translate("MainWindow", u"%v / %m", None))
        self.menu_File.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
        self.menu_Profile.setTitle(QCoreApplication.translate("MainWindow", u"Profile", None))
        self.menu_set_default_profile.setTitle(QCoreApplication.translate("MainWindow", u"Set Default", None))
        self.menu_settings.setTitle(QCoreApplication.translate("MainWindow", u"Settings", None))
        self.menu_theme.setTitle(QCoreApplication.translate("MainWindow", u"Theme", None))
        self.menu_Help.setTitle(QCoreApplication.translate("MainWindow", u"Help", None))
    # retranslateUi

